package com.wrj.driver.server.service.impl;

import cn.hutool.core.util.PhoneUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.wrj.driver.server.constant.Constants;
import com.wrj.driver.server.dto.LoginUserDto;
import com.wrj.driver.server.dto.SecurityUser;
import com.wrj.driver.server.entity.*;
import com.wrj.driver.server.exception.BusinessException;
import com.wrj.driver.server.mapper.SysUserMapper;
import com.wrj.driver.server.redis.RedisCache;
import com.wrj.driver.server.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.wrj.driver.server.util.*;
import com.wrj.driver.server.vo.LoginUserVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 用户表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements ISysUserService {

    @Autowired
    private RedisCache redisCache;

    @Value("${token.expireTime}")
    private Integer tokenExpireTime;

    @Value("${message.code.expireTime}")
    private Integer codeExpireTime;

    @Override
    public Boolean code(String phone) {

        if (!PhoneUtil.isPhone(phone)) {
            throw new BusinessException("手机号格式不正确");
        }

        if (redisCache.hasKey(Constants.MESSAGE_CODE_PREFIX + phone)){
            Long expire = redisCache.getExpire(Constants.MESSAGE_CODE_PREFIX + phone);
            throw new BusinessException("请在" + expire + "秒后重新获取验证码");
        }

        String code = SmsUtil.sendMessage(phone);
        redisCache.setCacheObject((Constants.MESSAGE_CODE_PREFIX + phone), code, codeExpireTime, TimeUnit.SECONDS);
        return Boolean.TRUE;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public LoginUserVo login(LoginUserDto loginUserDto) {
        SysUser tdSysUser = this.baseMapper.selectOne(new LambdaQueryWrapper<SysUser>()
                .eq(SysUser::getPhone, loginUserDto.getPhone()));

        //如果需要验证，验证用户是否存在
        if (loginUserDto.getIsCheck() != null && loginUserDto.getIsCheck()){
            if (tdSysUser == null){
                throw new BusinessException("您无权限登录！");
            }
        }

        //校验验证码 如果是默认验证码则不需要校验
        if (!StrUtil.equals(Constants.DEFAULT_MESSAGE_CODE,loginUserDto.getCode())){
            String code = redisCache.getCacheObject(Constants.MESSAGE_CODE_PREFIX + loginUserDto.getPhone());

            if (StrUtil.isBlank(code)){
                throw new BusinessException("验证码已过期，请重新获取");
            }

            if (!StrUtil.equals(code,loginUserDto.getCode())){
                throw new BusinessException("验证码错误");
            }
        }


        //todo 用户基础信息填写这里还要完善一下
        if (tdSysUser == null) {

            tdSysUser = new SysUser()
                    .setUserName("飞行操作员")
                    .setAvatar("")
                    .setPhone(loginUserDto.getPhone())
                    .setCreateTime(LocalDateTime.now());

            int insert = this.getBaseMapper().insert(tdSysUser);
            if (insert != 1) {
                throw new BusinessException("用户注册异常");
            }
        }
        //清除上一个人登陆的token信息
        String oldToken = redisCache.getCacheObject(Constants.PHONE_PREFIX + tdSysUser.getPhone());
        if (StrUtil.isNotBlank(oldToken)){
            redisCache.deleteObject(Constants.PHONE_PREFIX + tdSysUser.getPhone());
            redisCache.deleteObject(oldToken);
        }

        String token = UUID.randomUUID().toString();
        LoginUserVo loginUserVo = new LoginUserVo();
        BeanUtils.copyProperties(tdSysUser, loginUserVo);
        loginUserVo.setToken(token);
        loginUserVo.setDays(DateTimeUtil.getIntervalDays(loginUserVo.getCreateTime(),LocalDateTime.now()) + 1);
        // 存入redis
        redisCache.setCacheObject(token, tdSysUser, tokenExpireTime, TimeUnit.DAYS);
        redisCache.setCacheObject(Constants.PHONE_PREFIX + tdSysUser.getPhone(), token, tokenExpireTime, TimeUnit.DAYS);
        return loginUserVo;
    }

    @Override
    public Long loginOut() {
        String token = TokenThreadUtil.getToken();
        if (StrUtil.isNotBlank(token)) {
            SysUser tdSysUser = redisCache.getCacheObject(token);
            if (Objects.nonNull(tdSysUser) && Objects.nonNull(tdSysUser.getPhone())){
                redisCache.deleteObject(Constants.PHONE_PREFIX + tdSysUser.getPhone());
            }

            redisCache.deleteObject(token);
            return tdSysUser.getUserId();
        }
        return null;
    }

    @Override
    public LoginUserVo queryUserMessage() {
        SecurityUser loginUser = SecurityUtil.getLoginUser();

        LambdaQueryWrapper<SysUser> cond = new LambdaQueryWrapper<SysUser>()
                .eq(SysUser::getUserId, loginUser.getUserId());

        SysUser tdSysUser = this.baseMapper.selectOne(cond);
        if (Objects.isNull(tdSysUser)){
            throw new BusinessException("用户不存在或者已被删除");
        }

        LoginUserVo loginUserVo = new LoginUserVo();
        BeanUtils.copyProperties(tdSysUser, loginUserVo);
        loginUserVo.setDays(DateTimeUtil.getIntervalDays(loginUserVo.getCreateTime(),LocalDateTime.now()) + 1);

        return loginUserVo;
    }

}
