package com.wrj.driver.server.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.wrj.driver.server.constant.Constants;
import com.wrj.driver.server.dto.SysConfigDto;
import com.wrj.driver.server.entity.WrjSysConfig;
import com.wrj.driver.server.exception.BusinessException;
import com.wrj.driver.server.mapper.WrjSysConfigMapper;
import com.wrj.driver.server.redis.RedisCache;
import com.wrj.driver.server.service.IWrjSysConfigService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 系统配置表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
@Slf4j
public class WrjSysConfigServiceImpl extends ServiceImpl<WrjSysConfigMapper, WrjSysConfig> implements IWrjSysConfigService {

    @Autowired
    private RedisCache redisCache;

    @Override
    public WrjSysConfig queryConfigByKey(String configKey, Integer driverTypeId) {
        log.info("获取系统配置参数，configKey：{}", configKey);

        WrjSysConfig config = redisCache.getCacheObject(Constants.CONFIG_PREFIX + configKey + "_" + driverTypeId);
        if (Objects.nonNull(config)){
            return config;
        }

        LambdaQueryWrapper<WrjSysConfig> queryWrapper = new QueryWrapper<WrjSysConfig>().lambda()
                .eq(WrjSysConfig::getDriverTypeId,driverTypeId)
                .eq(WrjSysConfig::getConfigKey, configKey)
                .eq(WrjSysConfig::getIsActive, Constants.IS_ACTIVE_TRUE);
        config = getOne(queryWrapper);

        if (Objects.isNull(config)){
            return config;
        }

        //获取redis缓存时间
        WrjSysConfig tdSysConfig = null;
        if (StrUtil.equals(configKey,Constants.REDIS_EXPIRE_TIME)){
            tdSysConfig = config;
        }else {
            tdSysConfig = queryConfigByKey(Constants.REDIS_EXPIRE_TIME, Constants.DEFAULT_TYPE_ID);
        }

        if (Objects.isNull(tdSysConfig)) {
            log.info("config is not exist");
            throw new BusinessException("系统异常");
        }
        // 塞入redis中
        redisCache.setCacheObject(Constants.CONFIG_PREFIX + configKey + "_" + driverTypeId, config, Long.parseLong(tdSysConfig.getConfigValue()));

        return config;
    }

    @Override
    public List<WrjSysConfig> queryConfigByKey(String configKey) {
        log.info("获取系统配置参数，configKey：{}", configKey);

        LambdaQueryWrapper<WrjSysConfig> queryWrapper = new QueryWrapper<WrjSysConfig>().lambda()
                .eq(WrjSysConfig::getConfigKey, configKey)
                .eq(WrjSysConfig::getIsActive, Constants.IS_ACTIVE_TRUE);
       return list(queryWrapper);
    }

    @Override
    public String queryConfigValueByKey(String configKey, Integer driverTypeId) {
        log.info("获取系统配置参数，configKey：{}", configKey);
        WrjSysConfig config = queryConfigByKey(configKey,driverTypeId);
        if(null == config) {
            return null;
        }
        return config.getConfigValue();
    }

    @Override
    public Map<String, WrjSysConfig> queryConfigByKeys(List<String> configKeyList, Integer driverTypeId) {
        log.info("获取系统配置参数，configKeyList：{}", configKeyList);
        LambdaQueryWrapper<WrjSysConfig> queryWrapper = new QueryWrapper<WrjSysConfig>().lambda()
                .eq(WrjSysConfig::getDriverTypeId,driverTypeId)
                .in(WrjSysConfig::getConfigKey, configKeyList)
                .eq(WrjSysConfig::getIsActive, Constants.IS_ACTIVE_TRUE);
        List<WrjSysConfig> configList = list(queryWrapper);

        if(CollectionUtil.isEmpty(configList)) {
            return Collections.EMPTY_MAP;
        }
        return configList.stream().collect(Collectors.toMap(WrjSysConfig::getConfigKey, sysConfig -> sysConfig));
    }

    @Override
    public Boolean saveConfig(SysConfigDto configDto) {

        LambdaQueryWrapper<WrjSysConfig> queryWrapper = new QueryWrapper<WrjSysConfig>().lambda()
                .eq(WrjSysConfig::getDriverTypeId,configDto.getDriverTypeId())
                .eq(WrjSysConfig::getConfigKey, configDto.getConfigKey())
                .eq(WrjSysConfig::getIsActive, Constants.IS_ACTIVE_TRUE);
        WrjSysConfig config = getOne(queryWrapper);

        if (Objects.isNull(config)){
            config = new WrjSysConfig();
            BeanUtils.copyProperties(configDto, config);
            config.setCreateTime(LocalDateTime.now());
            config.setIsActive(Constants.IS_ACTIVE_TRUE);
            this.getBaseMapper().insert(config);
        } else {
            BeanUtils.copyProperties(configDto, config);
            this.getBaseMapper().updateConfig(config);
        }

        // 塞入redis中
        long timeout = 7200l;
        try{
            timeout = Long.parseLong(config.getConfigValue());
        } catch (Exception e){
            timeout = 7200l;
        }
        redisCache.setCacheObject(Constants.CONFIG_PREFIX + config.getConfigKey() + "_" + config.getDriverTypeId(), config, timeout);

        return true;
    }
}
